/*************************************************************/
/** MonochromeLib - monochrome graphic library for fx-9860G **/
/** MonochromeLib is free software                          **/
/**                                                         **/
/** @author Pierre "PierrotLL" Le Gall                      **/
/** @contact legallpierre89@gmail.com                       **/
/**                                                         **/
/** @file MonochromeLib.h                                   **/
/** Include header for MonochromeLib                        **/
/**                                                         **/
/** @date 06-17-2010                                        **/
/*************************************************************/

#ifndef MONOCHROMELIB
#define MONOCHROMELIB

/**************************/
/** Functions prototypes **/
/**************************/

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {Transparent=-1, White=0, LightG=1, DarkG=2, Black=3, LightXor=4, DarkXor=8, Xor=12} Color;

#define sgn(x)	(x<0?-1:1)
#define rnd(x)	(x-(int)x<.5?(int)x:(int)x+1)

char* vram_adress();

void pixel(int x, int y, Color);

void horizontal_line(int y, int x1, int x2, Color color);
void vertical_line(int x, int y1, int y2, Color color);
void line(int x1, int y1, int x2, int y2, Color color);

void rectangle(int x1, int y1, int x2, int y2, int border_width, Color border_color, Color fill_color);

void polygone(int *x, int *y, int nb_vertices, Color color);
void filled_polygone(int *x, int *y, int nb_vertices, Color color);

void circle(int x, int y, int radius, Color color);
void filled_circle(int x, int y, int radius, Color color);

void ellipse(int x, int y, int radius1, int radius2, Color color);
void ellipse_in_rect(int x1, int y1, int x2, int y2, Color color);
void filled_ellipse(int x, int y, int radius1, int radius2, Color color);
void filled_ellipse_in_rect(int x, int y, int radius1, int radius2, Color color);

void set_gray_buffers(void* light_buffer, void* dark_buffer);
void set_gray_swap(void* light_swap, void* dark_swap);
void* get_light_buffer();
void* get_dark_buffer();

void flip();

/*
void bmp_or(unsigned char *bmp, int x, int y, int width, int height);
void bmp_and(unsigned char *bmp, int x, int y, int width, int height);
void bmp_xor(unsigned char *bmp, int x, int y, int width, int height);
void bmp_or_cl(unsigned char *bmp, int x, int y, int width, int height);
void bmp_and_cl(unsigned char *bmp, int x, int y, int width, int height);
void bmp_xor_cl(unsigned char *bmp, int x, int y, int width, int height);

void bmp_8_or(unsigned char *bmp, int x, int y);
void bmp_8_and(unsigned char *bmp, int x, int y);
void bmp_8_xor(unsigned char *bmp, int x, int y);
void bmp_8_or_cl(unsigned char *bmp, int x, int y);
void bmp_8_and_cl(unsigned char *bmp, int x, int y);
void bmp_8_xor_cl(unsigned char *bmp, int x, int y);

void bmp_16_or(unsigned short *bmp, int x, int y);
void bmp_16_and(unsigned short *bmp, int x, int y);
void bmp_16_xor(unsigned short *bmp, int x, int y);
void bmp_16_or_cl(unsigned short *bmp, int x, int y);
void bmp_16_and_cl(unsigned short *bmp, int x, int y);
void bmp_16_xor_cl(unsigned short *bmp, int x, int y);*/

#ifdef __cplusplus
}
#endif


#endif //MONOCHROMELIB
