/*
	Revolution-FX - Hardware Library for the fx-9860SD, fx-9860G, fx-9860AU, Graph85 and Graph85SD
	Written by kucalc
	Modified by Kristaba
	
	Current Version: v1.0 GNU Toolchain
	Contact: kucalc@gmail.com
	Homepage: http://revolution-fx.sourceforge.net
	Official Support: http://www.casiocalc.org
	Wiki: http://revolution-fx.sourceforge.net/wiki
	
	This work is licensed under the Creative Commons Attribution-Noncommercial-Share Alike 3.0 License. 
	To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a 
	letter to Creative Commons, 171 Second Street, Suite 300, San Francisco, California, 94105, USA.
*/	


#ifndef REVOLUTION_H
#define REVOLUTION_H


/*
	Useful Definitions
*/

#define VRAM (unsigned char*)0x8800498D
#define SDK_VRAM (unsigned char *)0x88004991
#define REAL_VRAM ((unsigned char *)Syscall( 0, 0, 0, 0, 0x0135 ))

int Syscall( int R4, int R5, int R6, int R7, int FNo );

#ifndef _SIZE_T
#define _SIZE_T
typedef unsigned int size_t;
#endif

void *calloc(size_t, size_t);
void free(void *);
void *malloc(size_t);
void *realloc(void *, size_t);

void *memcpy(void *, const void *, size_t);
void *memset(void *, int, size_t);



/*
	Definitions for the SuperH timers
*/

/*
	Functions for accessing the RTC. The SuperH sees these values
	as BCD (binary coded decimals).
	Example:
		thousands = 2;
		hundreds = 0;
		tens = 0;
		ones = 7;
	Will give you: 2007 in decimal
*/

void RTCReset();
void RTCStart();
void RTCStop();
void RTCSetYear(unsigned char thousands, unsigned char hundreds, unsigned char tens, unsigned char ones);
void RTCSetMonth(unsigned char tens, unsigned char ones);
void RTCSetDayOfMonth(unsigned char tens, unsigned char ones);
void RTCSetDayOfWeek(unsigned char ones);
void RTCSetHour(unsigned char tens, unsigned char ones);
void RTCSetMinute(unsigned char tens, unsigned char ones);
void RTCSetSecond(unsigned char tens, unsigned char ones);
int RTCReadYear();
int RTCReadMonth();
int RTCReadDayOfMonth();
int RTCReadDayOfWeek();
int RTCReadHour();
int RTCReadMinute();
int RTCReadSecond();

/*
	Functions for reading the ports of the SuperH processor.
	Ports A~N (excluding I) and the SCP (Serial Communications Port)
*/

unsigned char ReadPort_A();
unsigned char ReadPort_B();
unsigned char ReadPort_C();
unsigned char ReadPort_D();
unsigned char ReadPort_E();
unsigned char ReadPort_F();
unsigned char ReadPort_G();
unsigned char ReadPort_H();
unsigned char ReadPort_J();
unsigned char ReadPort_K();
unsigned char ReadPort_L();
unsigned char ReadPort_M();
unsigned char ReadPort_N();
unsigned char ReadPort_SCP();

/*
	Functions for the graphics component of Revolution-FX.
	Grayscales, contrast, fast screen drawing
*/

void DrawAll(unsigned long *buffer);
void GrayInit(int countdown, int countdown2);
void GrayLinkBuffers(unsigned long *buffer1, unsigned long *buffer2);
void GrayInit3(int countdown, int countdown2, int countdown3);
void GrayLinkBuffers3(unsigned long *buffer1, unsigned long *buffer2, unsigned long *buffer3);
void GrayEnd();
void CopyVRAM(unsigned int *buffer);
void Change_Contrast(char contrast);

/*
	Functions for changing the CPU speed.
*/

void CPUSpeedNormal();
void CPUSpeedDouble();
void CPUSpeedTriple();
void CPUSpeedQuadruple();

/* 
	Functions for accessing and controlling the serial port
*/

void Init_SerialPort();
void Start_SerialPort();
void SerialTransmit(unsigned char value);
unsigned char SerialReceive();
void Stop_SerialPort();

/*
	Misc. Hardware Functions
*/

void Reset_Calc();

/*
	Functions for Revolution-FX graphics library
*/

#define NO_TRANS		0
#define TRANS		1

#define WHITE		0
#define BLACK		1
#define INVERT		2

#define FONT_FIVE		0
#define FONT_SIX		1
#define FONT_SEVEN	2
#define FONT_NINE		3

char GetPixel(unsigned char x, unsigned char y, unsigned char *buffer);
void PlotPixel(unsigned char x, unsigned char y, unsigned char *buffer, char draw_type);
void DrawLine(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char *buffer, char color);
void ClearArea(unsigned char left, unsigned char top, unsigned char right, unsigned char bottom, unsigned char *buffer);
void InvertArea(unsigned char left, unsigned char top, unsigned char right, unsigned char bottom, unsigned char *buffer);
void DrawSprite(unsigned char x_coor, unsigned char y_coor, unsigned char width, unsigned char height, unsigned char sprite_width, const unsigned char *sprite, unsigned char *buffer, char check_trans);
void CoolText(unsigned char x, unsigned char y, unsigned char font, char *str, unsigned char *buffer);

/*
	Functions for Revolution-FX keyboard library
*/

#define K_F1	0x69
#define K_F2	0x59
#define K_F3	0x49
#define K_F4	0x39
#define K_F5	0x29
#define K_F6	0x19

#define K_SHIFT	0x68
#define K_OPTN	0x58
#define K_VARS	0x48
#define K_MENU	0x38
#define K_LEFT	0x28
#define K_UP	0x18

#define K_ALPHA	0x67
#define K_SQR	0x57
#define K_EXPO	0x47
#define K_EXIT	0x37
#define K_DOWN	0x27
#define K_RIGHT	0x17

#define K_THETA	0x66
#define K_LOG	0x56
#define K_LN	0x46
#define K_SIN	0x36
#define K_COS	0x26
#define K_TAN	0x16

#define K_FRAC	0x65
#define K_FD	0x55
#define K_LPAR	0x45
#define K_RPAR	0x35
#define K_COMMA	0x25
#define K_STORE	0x15

#define K_7	0x64
#define K_8	0x54
#define K_9	0x44
#define K_DEL	0x34

#define K_4	0x63
#define K_5	0x53
#define K_6	0x43
#define K_MULT	0x33
#define K_DIV	0x23

#define K_1	0x62
#define K_2	0x52
#define K_3	0x42
#define K_PLUS	0x32
#define K_MINUS	0x22

#define K_0	0x61
#define K_DOT	0x51
#define K_EXP	0x41
#define K_NEG	0x31
#define K_EXE	0x21

#define K_AC	0x00

int FastKeyDown(unsigned char code);
unsigned char FastGetKey();

/*
	Non-hardware related functions
*/

int string_input(int x, int y, char string[]);

#endif
